@@ -57,11 +57,9 @@ pd_importance.default <- function(object, ...) {
 #' @describeIn pd_importance PD based feature importance from "hstats" object.
 #' @export
 pd_importance.hstats <- function(object, normalize = TRUE, squared = TRUE, 
-                                 sort = TRUE, top_m = 15L, eps = 1e-8, plot = FALSE, 
-                                 fill = "#2b51a1", ...) {
-  num <- with(
-    object, matrix(nrow = length(v), ncol = K, dimnames = list(v, pred_names))
-  )
+                                 sort = TRUE, top_m = 15L, zero = TRUE, eps = 1e-8, 
+                                 plot = FALSE, fill = "#2b51a1", ...) {
+  num <- init_numerator(object, way = 1L)
   for (z in object[["v"]]) {
     num[z, ] <- with(object, wcolMeans((f - F_not_j[[z]])^2, w = w))
   }
@@ -71,7 +69,8 @@ pd_importance.hstats <- function(object, normalize = TRUE, squared = TRUE,
     normalize = normalize, 
     squared = squared, 
     sort = sort, 
-    top_m = top_m, 
+    top_m = top_m,
+    zero = zero,
     eps = eps
   )
   if (plot) plot_stat(out, fill = fill, ...) else out
