@@ -238,6 +238,9 @@ test_that("postprocess() works for matrix input", {
     postprocess(num = num, denom = 1:2, sort = FALSE), 
       num / cbind(c(1, 1, 1), c(2, 2, 2))
   )
+  
+  expect_equal(postprocess(num = cbind(0:1, 0:1), zero = FALSE), rbind(c(1, 1)))
+  expect_null(postprocess(num = cbind(0, 0), zero = FALSE))
 })
 
 test_that("postprocess() works for vector input", {
@@ -249,6 +252,9 @@ test_that("postprocess() works for vector input", {
   expect_equal(postprocess(num = num, sort = FALSE), num)
   expect_equal(postprocess(num = num, sort = FALSE, top_m = 2), num[1:2])
   expect_equal(postprocess(num = num, squared = FALSE), sqrt(num[3:1]))
+  
+  expect_equal(postprocess(num = 0:1, denom = c(2, 2), zero = FALSE), 0.5)
+  expect_null(postprocess(num = 0, zero = FALSE))
 })
 
 test_that(".zap_small() works for vector input", {
